/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 366 (Exercise B.3.5)
**
*/

new;
library rpb,pgraph;

cls;

__output = 0;

/*
**> Question 1.a
**
*/


print;
print "=====================  Question 1.c  =======================================";

n = 2~3~5~10;
let rho = 0.10 0.20 0.50 0.70 0.90;
lambda_max = 1 + (n-1) .* rho;
pi_min = lambda_max ./ n;

results = (error(0)~n)|100*(rho~pi_min);

print ftosa(Results',"%lf",3,1);

print;
print "=====================  Question 1.d  =======================================";

let sigma = 0.18 0.20 0.22 0.25;
let rho =   1.00
            0.50 1.00
            0.20 0.30 1.00
            0.10 0.40 0.30 1.00;
rho = xpnd(rho);
call invpd(rho);
cov = rho .* sigma .* sigma';
n = rows(cov);

proc f_min(x);
  retp(x'*cov*x);
endp;

proc f_max(x);
  retp(-x'*cov*x);
endp;

proc EqProc(x);
  retp(x'x - 1.0);
endp;

x0 = ones(n,1)/n;

_sqp_EqProc = &EqProc;
{x_max,fmin,lagr,retcode} = sqpSolve(&f_max,x0);
{x_min,fmin,lagr,retcode} = sqpSolve(&f_min,x0);

{va,ve,databuf} = regPCA(cov,0);

print ftosa(ve|va',"%lf",3,4);
print ftosa(x_max~x_min,"%lf",3,4);

print;
print "=====================  Question 1.e  =======================================";

{va,ve,databuf} = regPCA(rho,0);

print ftosa(ve|va',"%lf",3,4);

pi1 = va[1]/sumc(va);

rho_bar = meanc(packr(vec(diagrv(rho,error(0)))));
lambda_min = 1 + (n-1) * rho_bar;
pi_min = lambda_min/n;
print;
print "pi(1) = ";
print ftosa(100*(pi1~pi_min),"%lf",3,2);
print;
print ftos(100*rho_bar,"rho(bar) = %lf",3,2);

print;
print "=====================  Question 1.f  =======================================";

{va,ve,databuf} = regPCA(cov,0);
A = ve[.,1 2];
Omega = diagrv(zeros(2,2),va[1:2]);
D = cov - A*Omega*A';
print ftosa(1e4*cov,"%lf",3,4);
print ftosa(1e4*D,"%lf",3,4);

A = ve[.,3 4];
Omega = diagrv(zeros(2,2),va[3:4]);
D = A*Omega*A';
print ftosa(1e4*D,"%lf",3,4);

/*
**> Question 2
**
*/

let sigma = 
          14.400 
          18.815 
          20.543 
          20.809 
          20.792;

sigma = sigma / 1e4;

let rho[5,5] =  
         100.000           80.316           67.518           59.340           51.114 
          80.316          100.000           96.176           91.023           84.085 
          67.518           96.176          100.000           98.467           94.442 
          59.340           91.023           98.467          100.000           98.557 
          51.114           84.085           94.442           98.557          100.000; 

rho = rho / 100;

let Rt = 
           0.493 
           0.626 
           0.960 
           1.349 
           1.794;

Rt = Rt / 100;

let tau = 1 3 5 7 10; 

cov = rho .* sigma .* sigma';

print;
print "=====================  Question 2.a  =======================================";

alpha = 0.99;

Ct = ones(5,1);
t = tau;
Bt = exp(-t .* Rt);
Dt = t;
delta = -Ct .* Dt .* Bt;
VaR = cdfni(alpha) * sqrt(delta' * cov * delta);

mr = cdfni(alpha) .* (cov * delta) ./ sqrt(delta' * cov * delta);
rc = delta .* cdfni(alpha) .* (cov * delta) ./ sqrt(delta' * cov * delta);
pc = rc/VaR;

Results = delta~100*mr~100*rc~100*pc | (error(0)~error(0)~100*VaR~error(0));
print ftosa(Results,"%lf",3,2);

nB = ones(5,1);
Ct = eye(5);

A = - Dt .* Bt .* Ct;
A = A';
VaR = cdfni(alpha) * sqrt(nB' * (A * cov * A') * nB); 
mr = cdfni(alpha) .* (A * cov * A' * nB) ./ sqrt(nB' * (A * cov * A') * nB); 
rc = nB .* cdfni(alpha) .* (A * cov * A' * nB) ./ sqrt(nB' * (A * cov * A') * nB); 
pc = rc/VaR;

Results = nB~100*mr~100*rc~100*pc | (error(0)~error(0)~100*VaR~error(0));
print ftosa(Results,"%lf",3,2);

Pt = sumr(Ct .* Bt);
w =  (nB .* Pt)/sumc(nB .* Pt);
mr = rc ./ w;
Results = 100*w~100*mr~100*rc~100*pc | (error(0)~error(0)~100*VaR~error(0));
print ftosa(Results,"%lf",3,2);

print;
print "=====================  Question 2.b  =======================================";

{lambda,V,databuf} = regPCA(cov,0);
Pi_ = lambda / sumc(lambda);
Pi_c = cumsumc(Pi_);
Results = V|(1e6*lambda')|100*Pi_'|100*Pi_c';
print ftosa(Results,"%lf",3,4);

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pcross = 1; _pltype = 6|3|1; _plwidth = 10;
  _pcolor = 1|4|2; _pstype = 8; _plctrl = 1;
  xtics(0,10,1,2);
  _plegstr = "Factor #1\000Factor #2\000Factor #3";
  _plegctl = {2 5 5 1};
  graphprt("-c=2 -cf=app2-3-5-2.eps");
  xy(tau,V[.,1 2 3]);

print;
print "=====================  Question 2.c  =======================================";

wR = delta;

{vol,mr,rc,pc} = Compute_Risk_Contribution(wR,cov);

VaR = cdfni(alpha) * vol;
mr = cdfni(alpha) * mr;
rc = cdfni(alpha) * rc;
Results = wR~100*mr~100*rc~100*pc | (error(0)~error(0)~100*VaR~error(0));
print ftosa(Results,"%lf",3,2);

A = V;
Omega = diagrv(zeros(5,5),lambda);
D = zeros(5,5);

{vol,y,mr,rc,prc} = Compute_RC_Factor(wR,cov,A,Omega,D);
VaR = cdfni(alpha) * vol;
mr = cdfni(alpha) * mr;
rc = cdfni(alpha) * rc;
Results = y~100*mr~100*rc~100*prc | (error(0)~error(0)~100*VaR~error(0));
print ftosa(Results,"%lf",3,2);

print;
print "=====================  Question 2.d  =======================================";

{x,vol,mr,rc,pc} = Compute_ERC_portfolio(0,cov);
nB = -100*x ./ delta;
nB = nB / nB[5];

Ct = eye(5);

A = - Dt .* Bt .* Ct;
A = A';
VaR = cdfni(alpha) * sqrt(nB' * (A * cov * A') * nB); 
mr = cdfni(alpha) .* (A * cov * A' * nB) ./ sqrt(nB' * (A * cov * A') * nB); 
rc = nB .* cdfni(alpha) .* (A * cov * A' * nB) ./ sqrt(nB' * (A * cov * A') * nB); 
pc = rc/VaR;

Results = nB~100*mr~100*rc~100*pc | (error(0)~error(0)~100*VaR~error(0));
print ftosa(Results,"%lf",3,2);

wR = nB .* delta;

A = V;
Omega = diagrv(zeros(5,5),lambda);
D = zeros(5,5);

{vol,y,mr,rc,prc} = Compute_RC_Factor(wR,cov,A,Omega,D);
VaR = cdfni(alpha) * vol;
mr = cdfni(alpha) * mr;
rc = cdfni(alpha) * rc;
Results = y~100*mr~100*rc~100*prc | (error(0)~error(0)~100*VaR~error(0));
print ftosa(Results,"%lf",3,2);

print;
print "=====================  Question 2.e  =======================================";

Results = zeros(5,5);

i = 1;
do until i > 5;
  nB = V[.,i];
  wR = nB .* delta;
  A = V;
  Omega = diagrv(zeros(5,5),lambda);
  D = zeros(5,5);

  {vol,y,mr,rc,prc} = Compute_RC_Factor(wR,cov,A,Omega,D);
  VaR = cdfni(alpha) * vol;
  mr = cdfni(alpha) * mr;
  rc = cdfni(alpha) * rc;
  Results[.,i] = 100*prc;
  i = i + 1;
endo;

print ftosa(Results,"%lf",3,2);
