/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 370 (Exercise B.3.7)
**
*/

new;
library rpb,pgraph;

cls;

__output = 0;

/*
**> Question 1
**
*/

print;
print "=====================  Question 1.b  =======================================";

sigma1 = 0.20; sigma2 = 0.10;
rho = 0.5;

sigma = sigma1|sigma2;
cov = xpnd(1|rho|1) .* sigma .* sigma';

x1 = rndn(1,1);
x2 = rndn(1,1);
x = x1 | x2;

{vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov);

vol = sqrt(x1^2 .* sigma1^2 + 2 * x1 .* x2 .* rho .* sigma1 .* sigma2 + x2^2 .* sigma2^2);
RC1 = (x1^2 .* sigma1^2 + x1 .* x2 .* rho .* sigma1 .* sigma2) ./ vol;
RC2 = (x2^2 .* sigma2^2 + x1 .* x2 .* rho .* sigma1 .* sigma2) ./ vol;

print rc~(rc1|rc2);

print;
print "=====================  Question 1.c  =======================================";

sigma1 = 0.18; 
sigma2 = 0.10;
let rho[1,4] = 0.9 0.5 0.0 -0.9;

x1 = 1.0;
x2 = seqa(-3,6/100,101);

vol = sqrt(x1^2 .* sigma1^2 + 2 * x1 .* x2 .* rho .* sigma1 .* sigma2 + x2^2 .* sigma2^2);
RC1 = (x1^2 .* sigma1^2 + x1 .* x2 .* rho .* sigma1 .* sigma2) ./ vol;
RC2 = (x2^2 .* sigma2^2 + x1 .* x2 .* rho .* sigma1 .* sigma2) ./ vol;

x2_star = -x1 .* rho .* sigma1 ./ sigma2;
print ftosa(x2_star,"%lf",3,2);

graphset;
  begwind;
  window(2,2,0);

    fonts("simplex simgrma");
    _pdate = ""; _pnum = 2; _pframe = 0;
    _pcross = 1; _pltype = 6|3; _plwidth = 10;
    _pcolor = 1|4;
    xtics(-3,3,1,2);
    ytics(-0.10,0.30,0.05,2);
    _paxht = 0.20; _pnumht = 0.20; _ptitlht = 0.275;

  i = 1;
  setwind(i);
    title("\216\202r\201 = 90%");
    xy(x2,RC1[.,i]~RC2[.,i]);

  i = 2;
  setwind(i);
    title("\216\202r\201 = 50%");
    xy(x2,RC1[.,i]~RC2[.,i]);

  i = 3;
  setwind(i);
    title("\216\202r\201 = 0%");
    xy(x2,RC1[.,i]~RC2[.,i]);

  i = 4;
  setwind(i);
    title("\216\202r\201 = -90%");
    xy(x2,RC1[.,i]~RC2[.,i]);

    graphprt("-c=2 -cf=app2-3-7-1.eps");

  endwind;

/*
**> Question 2
**
*/

let sigma = 0.20 0.25 0.30 0.35 0.10 0.10;
let rho =   1.00
            0.60 1.00
            0.50 0.40 1.00
            0.30 0.20 0.30 1.00
            0.60 0.50 0.20 0.20 1.00
            0.50 0.40 0.30 0.30 0.40 1.00;
rho = xpnd(rho);
cov = rho .* sigma .* sigma';
call invpd(cov);

let x = 1.00 -1.00 1.00 -1.00 1.00 -1.00;
{vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov);

print;
print "=====================  Question 2.a  =======================================";

results = (x~mr~rc~prc)|(error(0)~error(0)~vol~error(0));
print ftosa(100*results,"%lf",3,2);

let A[3,6] =  1.00 -1.00 0.00  0.00  0.00  0.00
              0.00 0.00  1.00 -1.00  0.00  0.00
              0.00 0.00  0.00  0.00  1.00 -1.00; 
cov2 = A*cov*A';
sigma2 = sqrt(diag(cov2));
rho2 = cov2 ./ sigma2 ./ sigma2';

print;
print "=====================  Question 2.b  =======================================";

print ftosa(100*sigma2',"%lf",3,2);
print ftosa(100*rho2,"%lf",3,2);

rho3 = eye(3);
cov3 = rho3 .* sigma2 .* sigma2';

let x = 1.00 1.00 1.00;
{vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov2);
results = (x~mr~rc~prc)|(error(0)~error(0)~vol~error(0));
print ftosa(100*results,"%lf",3,2);

print;
print "=====================  Question 2.c  =======================================";

{x,vol,mr,rc,prc} = Compute_ERC_Portfolio(0,cov2);
results = (x~mr~rc~prc)|(error(0)~error(0)~vol~error(0));
print ftosa(100*results,"%lf",3,2);
x2 = x;

{x,vol,mr,rc,prc} = Compute_ERC_Portfolio(0,cov3);
results = (x~mr~rc~prc)|(error(0)~error(0)~vol~error(0));
print ftosa(100*results,"%lf",3,2);
x3 = x;

print;
print "=====================  Question 2.d  =======================================";

x = x2 .*. (1|-1);
{vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov);
results = (x~mr~rc~prc)|(error(0)~error(0)~vol~error(0));
print ftosa(100*results,"%lf",3,2);

print;
print "=====================  Question 2.e  =======================================";

proc func(x);
  local e,vol,mr,rc,prc,u;

  let e = 1 -1 1 -1 1 -1;
  x = e .* sqrt(x^2);

  {vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov);
  u = (rc[1] - 0.04) |
      (rc[2] - rc[1]) |
      (rc[3] - rc[4]) |
      (rc[5] - rc[6]) |
      (rc[1] + rc[2] - rc[3] - rc[4]) |
      (rc[5] + rc[6] - rc[3] - rc[4]) |
      (rc[1] + rc[2] - rc[5] - rc[6]) ;

  retp(sumc(u^2));
endp;

_sqp_A = ones(1,6); _sqp_B = 0;
_sqp_Bounds = -3~3;
_sqp_DirTol = 1e-7;
x0 = x;
{x,fmin,grd,retcode} = sqpsolve(&func,x0);
let e = 1 -1 1 -1 1 -1;
x = e .* sqrt(x^2);
{vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov);
results = (x~mr~rc~prc)|(error(0)~error(0)~vol~error(0));
print ftosa(100*results,"%lf",3,2);

A = diagrv(zeros(6,6),e);
cov2 = A*cov*A';

{x,vol,mr,rc,prc} = Compute_ERC_Portfolio(0,cov2);
{vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov2);
results = (x~mr~rc~prc)|(error(0)~error(0)~vol~error(0));
print ftosa(100*results,"%lf",3,2);

x = inv(A)*x;
{vol,mr,rc,prc} = Compute_Risk_Contribution(x,cov);
results = (x~mr~rc~prc)|(error(0)~error(0)~vol~error(0));
print ftosa(100*results,"%lf",3,2);







