/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 13 (Figure 1.4)
**
*/

new;
library rpb,pgraph;

cls;

let mu =    0.05 0.06 0.08 0.06;
let sigma = 0.15 0.20 0.25 0.30;
n = rows(sigma);
let rho =   1.00 
            0.10   1.00
            0.40   0.70   1.00
            0.50   0.40   0.80 1.00;
rho = xpnd(rho);
call invpd(rho);

cov = rho .* sigma .* sigma';
A = ones(1,n); B = 1;

Bounds = (-10~10) .* ones(n,1);
gamma_ = seqa(-2,1/100,600);
{x0,mu0,sigma0,gamma0,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,gamma_,0);  

r = 0.015;

SR = (mu0-r)./sigma0;
indx = maxindc(SR);
mu1 = mu0[indx]; sigma1 = sigma0[indx]; SR1 = SR[indx];
sigma2 = 0.40; mu2 = r + SR1 * sigma2;

{x3,mu3,sigma3,gamma3,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,0.35,2);  
SR3 = (mu3 - r)/sigma3;
sigma4 = 0.40; mu4 = r + SR3 * sigma4;

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6; _plwidth = 10; _pcolor = 1;
  xtics(0,40,5,2);
  ytics(0,12,2,0);
  xlabel("\216Volatility (in %)");
  ylabel("\216Expected return (in %)");
  _psym = 0~100*r~11~5~5~1~10 |
          100*sigma1~100*mu1~11~5~5~1~10 |
          100*sigma1~100*r~11~5~5~1~10 |
          100*sigma3~100*mu3~11~4.4~8~1~10 ;

  _pline = 1~3~0~100*r~100*sigma2~100*mu2~0~2~10 |
           1~3~0~100*r~100*sigma4~100*mu4~0~8~10 |
           1~3~100*sigma1~100*r~100*sigma1~100*mu1~0~2~10 |
           1~3~0~100*r~100*sigma1~100*r~1~2~10 |
           4~3~0~100*r~10~0~0.69~2~10 |
           4~3~0~100*r~9.50~0~0.69~2~10 ;

  _pmsgstr = "Risk-free asset\000Tangency portfolio\000\202Q\201\000Capital market line\000Suboptimal market line\000A";
  _pmsgctl =   0.5~100*r-0.50~0.12~0~1~5~7 |
               100*sigma1-9.5~100*mu1+0.1~0.12~0~1~5~7 |
               10.0~2.75~0.12~0~1~2~7 |
               100*sigma1+5~100*mu1+2.10~0.12~40~1~2~7 |
               100*sigma3-15~100*mu3-4.65~0.12~34~1~8~7 |
               100*(sigma3-0.015)~100*(mu3+0.0015)~0.18~0~1~8~7 ;  

  graphprt("-c=2 -cf=chap1-1-2-1.eps");
  xy(100*sigma0,100*mu0);


Bounds = (-3~3) .* ones(n,1);
gamma_ = seqa(0.2,2/10000,10000);

{x0,mu0,sigma0,gamma0,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,gamma_,0);  
SR = (mu0-r)./sigma0;
indx = maxindc(SR);
x1 = x0[.,indx]; mu1 = mu0[indx]; sigma1 = sigma0[indx]; SR1 = SR[indx]; gamma1 = gamma0[indx];
theta1 = 180*arctan(SR1)/pi;

print ftosa(100*x1,"%lf",3,2);
print "------------------------------------------------------------------------------------";

print ftosa(100*(mu1|sigma1)|SR1|theta1,"%lf",3,2);
