/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 15 (Figure 1.5)
**
*/

new;
library rpb,pgraph;

cls;

let mu =    0.05 0.06 0.08 0.06;
let sigma = 0.15 0.20 0.25 0.30;
n = rows(Sigma);
let rho =   1.00 
            0.10   1.00
            0.40   0.70   1.00
            0.50   0.40   0.80 1.00;
rho = xpnd(rho);
call invpd(rho);

cov = rho .* sigma .* sigma';
A = ones(1,n); B = 1;

r = 0.015;

/*
**> Tangency portfolio
**
*/

gamma_ = seqa(0.5,1/4000,4000);

Bounds = (-3~3) .* ones(n,1);
{x0,mu0,sigma0,gamma0,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,gamma_,0);  
SR = (mu0-r)./sigma0;
indx = maxindc(SR);
x0_SR = x0[.,indx]; mu0_SR = mu0[indx]; sigma0_SR = sigma0[indx]; phi0_SR = 1/gamma0[indx]; SR0_SR = SR[indx];

Bounds = (0~3) .* ones(n,1);
{x1,mu1,sigma1,gamma1,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,gamma_,0);  
SR = (mu1-r)./sigma1;
indx = maxindc(SR);
x1_SR = x1[.,indx]; mu1_SR = mu1[indx]; sigma1_SR = sigma1[indx]; phi1_SR = 1/gamma1[indx]; SR1_SR = SR[indx]; 

Bounds = (0~0.40) .* ones(n,1);
{x2,mu2,sigma2,gamma2,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,gamma_,0);  
SR = (mu2-r)./sigma2;
indx = maxindc(SR);
x2_SR = x2[.,indx]; mu2_SR = mu2[indx]; sigma2_SR = sigma2[indx]; phi2_SR = 1/gamma2[indx]; SR2_SR = SR[indx];

Bounds = (0.20~0.70) .* ones(n,1);
{x3,mu3,sigma3,gamma3,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,gamma_,0);  
SR = (mu3-r)./sigma3;
indx = maxindc(SR);
x3_SR = x3[.,indx]; mu3_SR = mu3[indx]; sigma3_SR = sigma3[indx]; phi3_SR = 1/gamma3[indx]; SR3_SR = SR[indx];

/*
**> Efficient frontier
**
*/

mu_tilde = mu|r;
cov_tilde = (cov~zeros(n,1)) | zeros(1,n+1);
A_tilde = ones(1,n+1);
B_tilde = 1;
Bounds_tilde = (-3~3) .* ones(n+1,1);

gamma_ = seqa(0,1/400,1700);

{x0,mu0,sigma0,gamma0,retcode} = Compute_MVO_Portfolio(mu_tilde,cov_tilde,A_tilde,B_tilde,0,0,Bounds_tilde,gamma_,0);
indx0 = minindc(abs(x0[n+1,.]'));
SR0 =(mu0[indx0]-r)./sigma0[indx0]; 

Bounds_tilde = (0~3) .* ones(n,1)|(-3~3);
{x1,mu1,sigma1,gamma1,retcode} = Compute_MVO_Portfolio(mu_tilde,cov_tilde,A_tilde,B_tilde,0,0,Bounds_tilde,gamma_,0);  
indx1 = minindc(abs(x1[n+1,.]'));
SR1 =(mu1[indx1]-r)./sigma1[indx1]; 

Bounds_tilde = (0~0.40) .* ones(n,1)|(-5~5);
{x2,mu2,sigma2,gamma2,retcode} = Compute_MVO_Portfolio(mu_tilde,cov_tilde,A_tilde,B_tilde,0,0,Bounds_tilde,gamma_,0);  
indx2 = minindc(abs(x2[n+1,.]'));
SR2 =(mu2[indx2]-r)./sigma2[indx2]; 

Bounds_tilde = (0.20~0.70) .* ones(n,1)|(-5~5);
{x3,mu3,sigma3,gamma3,retcode} = Compute_MVO_Portfolio(mu_tilde,cov_tilde,A_tilde,B_tilde,0,0,Bounds_tilde,gamma_,0);  
indx3 = minindc(abs(x3[n+1,.]'));
SR3 =(mu3[indx3]-r)./sigma3[indx3]; 

print ftosa(100*(x0[.,indx0]~(x0_SR|0)),"%lf",3,2);
print ftosa(100*(SR0_SR~SR0),"%lf",3,2);
print "--------------------------------------------";
print ftosa(100*(x1[.,indx1]~(x1_SR|0)),"%lf",3,2);
print ftosa(100*(SR1_SR~SR1),"%lf",3,2);
print "--------------------------------------------";
print ftosa(100*(x2[.,indx2]~(x2_SR|0)),"%lf",3,2);
print ftosa(100*(SR2_SR~SR2),"%lf",3,2);
print "--------------------------------------------";
print ftosa(100*(x3[.,indx3]~(x3_SR|0)),"%lf",3,2);
print ftosa(100*(SR3_SR~SR3),"%lf",3,2);

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|1|3|6; _plwidth = 10|10|10|6; _pcolor = 1|2|4|3; _plctrl = 0|0|0|10; _psymsiz = 2.5; _pstype = 8;
  xtics(0,30,5,2);
  ytics(1,10,1,0);
  xlabel("\216Volatility (in %)");
  ylabel("\216Expected return (in %)");
  _psym = 100*sigma0[indx0]~100*mu0[indx0]~11~5~1~1~10 |
          100*sigma1[indx1]~100*mu1[indx1]~11~5~2~1~10 |
          100*sigma2[indx2]~100*mu2[indx2]~11~5~4~1~10 |
          100*sigma3[indx3]~100*mu3[indx3]~11~5~3~1~10 ;
  _plegstr = "No constraint\000x]i[ \202\62 \2010%\0000% \202\60 \201x]i[ \202\60 \20140%\00020% \202\60 \201x]i[ \202\60\201 70%";
  _plegctl = {2 5 6.0 2};
  graphprt("-c=2 -cf=chap1-1-2-2.eps");
  xy(100*(sigma0~sigma1~sigma2~sigma3),100*(mu0~mu1~mu2~mu3));


