/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 26 (Table 1.7)
**
*/

new;
library rpb,pgraph;

cls;

let mu =    0.05 0.06 0.08 0.06;
let sigma = 0.15 0.20 0.25 0.30;
n = rows(sigma);
let rho =   1.00 
            0.10   1.00
            0.40   0.70   1.00
            0.50   0.40   0.80 1.00;
rho = xpnd(rho);
call invpd(rho);

cov = rho .* sigma .* sigma';

Bounds = 0;

r = 0.03;
SR = 0.25;
let x0 = 0.40 0.30 0.20 0.10;

tau = 1;
Gamma_ = tau * cov;

let P[2,4] = 1 0  0 0
             0 1 -1 0;
let Q = 0.04 -0.01;
let Omega[2,2] = 0.10 0.0
                 0.0 0.05;
Omega = Omega^2;

nIters = 6;
mu_bar = zeros(4,nIters);
x_star = zeros(4,nIters);
vol_te = zeros(1,nIters);

i = 1;
{phi,mu_tilde,mu_cond} = Compute_BL_Risk_Premium(x0,cov,SR,r,Gamma_,P,Q,Omega);
mu_bar[.,i] = mu_tilde;
x_star[.,i] = x0;
vol_te[i] = 0.0;

i = 2;
{phi,mu_tilde,mu_bar[.,i]} = Compute_BL_Risk_Premium(x0,cov,SR,r,Gamma_,P,Q,Omega);
{x_star[.,i],mu_x,sigma_x,vol_te[.,i],retcode} = Compute_BL_Portfolio(x0,cov,SR,r,Gamma_,P,Q,Omega,Bounds);

i = 3;
let Q = 0.07 -0.01;
{phi,mu_tilde,mu_bar[.,i]} = Compute_BL_Risk_Premium(x0,cov,SR,r,Gamma_,P,Q,Omega);
{x_star[.,i],mu_x,sigma_x,vol_te[.,i],retcode} = Compute_BL_Portfolio(x0,cov,SR,r,Gamma_,P,Q,Omega,Bounds);

i = 4;
let Q = 0.04 -0.01;
let Omega[2,2] = 0.20 0.0
                 0.0 0.20;
Omega = Omega^2;
{phi,mu_tilde,mu_bar[.,i]} = Compute_BL_Risk_Premium(x0,cov,SR,r,Gamma_,P,Q,Omega);
{x_star[.,i],mu_x,sigma_x,vol_te[.,i],retcode} = Compute_BL_Portfolio(x0,cov,SR,r,Gamma_,P,Q,Omega,Bounds);

i = 5;
let Omega[2,2] = 0.10 0.0
                 0.0 0.05;
Omega = Omega^2;
tau = 0.10;
Gamma_ = tau * cov;
{phi,mu_tilde,mu_bar[.,i]} = Compute_BL_Risk_Premium(x0,cov,SR,r,Gamma_,P,Q,Omega);
{x_star[.,i],mu_x,sigma_x,vol_te[.,i],retcode} = Compute_BL_Portfolio(x0,cov,SR,r,Gamma_,P,Q,Omega,Bounds);

i = 6;
let Omega[2,2] = 0.10 0.0
                 0.0 0.05;
Omega = Omega^2;
tau = 0.01;
Gamma_ = tau * cov;
{phi,mu_tilde,mu_bar[.,i]} = Compute_BL_Risk_Premium(x0,cov,SR,r,Gamma_,P,Q,Omega);
{x_star[.,i],mu_x,sigma_x,vol_te[.,i],retcode} = Compute_BL_Portfolio(x0,cov,SR,r,Gamma_,P,Q,Omega,Bounds);

print ftosa(100*mu_bar,"%lf",3,2);
print ftosa(100*x_star,"%lf",3,2);
print;
print ftosa(100*vol_te,"%lf",3,2);
