/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 36 (Figure 1.12)
**
*/

new;
library rpb,pgraph,cml;

PathName = "data\\";
datFileName = PathName $+ "equity-indices-daily.dat";
Data = loadd(datFileName);
vnames = lower("" $+ getname(datFileName));

let string selected_names = "spx";
indx = indsav(selected_names,vnames);
Dates = Data[.,1];
nDates = rows(dates);
x = Data[.,indx];
Rdt = x ./ lag1(x) - 1.0;

BeginDate = 19991231;
LastDate = 20111231;
indx = selif(seqa(1,1,nDates), (Dates .>= BeginDate) .and (Dates .<= LastDate));
Rdt = Rdt[indx];
Dates = Dates[indx];
x = x[indx];
nDates = rows(dates);

vol0 = 100 * stdc(Rdt)*sqrt(260);

// Garch ML
p = 1; q = 1;
{theta,cl,cov,logl} = regGarch(Rdt,1,p,q,0);
vol1 = 100 * sqrt(_garch_h) * sqrt(260);

// EWMA
lambda = 0.90;
vol2 = 100 * EWMA_Volatility(Dates,x,lambda);

// 20 days volatility
vol3 = 100 * Rolling_Volatility(Dates,x,20);

x2 = vol1; y2 = vol2;
{x2,y2} = packr2(x2,y2);
indx2 = delif_closed_points(x2,y2,0.03);

x3 = vol1; y3 = vol3;
{x3,y3} = packr2(x3,y3);
indx3 = delif_closed_points(x3,y3,0.03);

graphset;
  window3u;

  fonts("simplex simgrma");

  _pdate = 0; _pframe = 0; _pnum = 2;
  _plwidth = 10; _pltype = 6; _pcolor = 1;
  _paxht = 0.25;  _ptitlht = 0.275;  _pnumht = 0.20; 

  xtics(1,nDates,261*2,2);
  let xlab = "2000" "02" "04" "06" "08" "10" "12";
  asclabel(xlab,0);

  _pcolor = 1;
  setwind(1);
    title("\216GARCH(1,1)");
    ytics(0,100,20,2);
    _pline = 1~3~0~vol0~nDates~vol0~1~5~10;
    xy(0,vol1);

  _pline = 0; _plctrl = -1; _pstype = 4;
  xtics(0,100,20,2); ytics(0,100,20,2);
  asclabel(0,0); 
  _pline = 1~3~0~0~100~100~1~0~10;

  _pcolor = 2;
  setwind(2);
    title("\216EMWA (\202l\201 = 0.90)");
    xlabel("\216GARCH");
    ylabel("\216EMWA");
    xy(x2[indx2],y2[indx2]);

  _pcolor = 4; 
  setwind(3);
    title("\21620 days volatility");
    xlabel("\216GARCH");
    ylabel("\21620D");
    xy(x3[indx3],y3[indx3]);

    graphprt("-c=2 -cf=chap1-2-1-5.eps");

  endwind;

