/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 54 (Figure 1.18)
**
*/

new;
library rpb,pgraph;

cls;

let mu =    0.05 0.06 0.08 0.06;
let sigma = 0.15 0.20 0.25 0.30;
n = rows(Sigma);
let rho =   1.00 
            0.10   1.00
            0.40   0.70   1.00
            0.50   0.40   0.80 1.00;
rho = xpnd(rho);
call invpd(rho);

cov = rho .* sigma .* sigma';

x0 = ones(n,1)/n;
gamma_ = 0.5;

A = ones(1,n); B = 1.0;

A1 = eye(n);
A2 = ones(n,1);

nIters = 250;
lambda1 = seqa(0,0.5/nIters,nIters);
lambda2 = seqa(0,0.5/nIters,nIters);
lambda3 = seqa(0,0.05/nIters,nIters);
lambda4 = seqa(0,0.05/nIters,nIters);

x1 = zeros(n,nIters);
x2 = zeros(n,nIters);
x3 = zeros(n,nIters);
x4 = zeros(n,nIters);


i = 1;
do until i > nIters;
  {x1[.,i],er,vol,retcode} = Compute_Ridge_Portfolio(x0,mu,cov,A,B,0,0,0,gamma_,lambda1[i],A1,0);
  {x2[.,i],er,vol,retcode} = Compute_Ridge_Portfolio(x0,mu,cov,0,0,0,0,0,gamma_,lambda2[i],A1,1);
  {x3[.,i],er,vol,retcode} = Compute_Lasso_Portfolio(x0,mu,cov,A,B,0,0,0,gamma_,lambda3[i],A2,0);
  {x4[.,i],er,vol,retcode} = Compute_Lasso_Portfolio(x0,mu,cov,0,0,0,0,0,gamma_,lambda4[i],A2,1);
  i = i + 1;
endo;

graphset;
  begwind;
  window(2,2,0);

  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _plwidth = 10; 
  _paxht = 0.25;  _ptitlht = 0.275;  _pnumht = 0.20; 
  _pltype = 6|1|3|6; _pcolor = 1|2|3|4; _pstype = 8; _plctrl = 0|0|0|20; _psymsiz = 5;

  _pcross = 1;

  setwind(1);
    title("\216Ridge (S)");
    ytics(-50,75,25,2);
    xtics(0,0.5,0.1,2);
    xlabel("\216\202l\201");
    xy(lambda1,100*x1');

  setwind(2);
    title("\216Ridge (D)");
    ytics(-40,120,40,2);
    xy(lambda2,100*x2');

  setwind(3);
    title("\216Lasso (S)");
    xtics(0,5,1,2);
    ytics(-50,75,25,2);
    xlabel("\216\202l\201 (in %)");
    xy(100*lambda3,100*x3');

  setwind(4);
    title("\216Lasso (D)");
    ytics(-40,120,40,2);
    _plegstr = "#1\000#2\000#3\000#4";
    _plegctl = {2 6 6.5 3.75};
    xy(100*lambda4,100*x4');

    graphprt("-c=2 -cf=chap1-2-3-8.eps");

  endwind;


