/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 104 (Figure 2.4)
**
*/

new;
library rpb,pgraph;

cls;

let sigma1 = 0.20 0.20 0.20 0.20;
let sigma2 = 0.20 0.60 0.20 0.60;

let b      = 0.60 0.60 0.90 0.90;
rho = seqa(-0.99,1.98/100,101);
nRho = rows(rho);

w1 = zeros(nRho,4);
w2 = zeros(nRho,4);
vol = zeros(nRho,4);
rc = zeros(nRho,4);

j = 1;
do until j > 4;

  {w1[.,j],w2[.,j]} = Compute_RB_2A(sigma1[j],sigma2[j],rho,b[j]);

  sigma = sigma1[j] | sigma2[j];

  i = 1;
  do until i > nRho;
    cov = xpnd(1|rho[i]|1) .* sigma .* sigma';
    x = w1[i,j] | w2[i,j];
    {vol[i,j],mr,tr,_rc_} = Compute_Risk_Contribution(x,cov);
    rc[i,j] = _rc_[1];
    i = i + 1;
  endo;

  j = j + 1;
endo;

rc = rc - b';
print maxc(maxc(abs(rc)));
x1 = 100*rho;
y1 = 100*w1;

let rho    = 0.00 0.00 0.90 0.90;
b = 0|seqa(1/101,1/101,100)|1;
nB = rows(b);

w1 = zeros(nB,4);
w2 = zeros(nB,4);
vol = zeros(nB,4);
rc = zeros(nB,4);

j = 1;
do until j > 4;

  {w1[.,j],w2[.,j]} = Compute_RB_2A(sigma1[j],sigma2[j],rho[j],b);

  sigma = sigma1[j] | sigma2[j];
  cov = xpnd(1|rho[j]|1) .* sigma .* sigma';

  i = 1;
  do until i > nB;
    x = w1[i,j] | w2[i,j];
    {vol[i,j],mr,tr,_rc_} = Compute_Risk_Contribution(x,cov);
    rc[i,j] = _rc_[1];
    i = i + 1;
  endo;

  j = j + 1;
endo;

rc = rc - b;
print maxc(maxc(abs(rc)));
x2 = 100*b;
y2 = 100*w1;

graphset;
  begwind;
  window(2,2,0);

  fonts("simplex simgrma");
  _pdate = ""; _pframe = 0; _pnum = 2;
  _pltype = 6|3; _pcolor = 1|4; _plwidth = 10;
  _paxht = 0.25;  _ptitlht = 0.275;  _pnumht = 0.20;

  xtics(-100,100,50,2);
  xlabel("\216\202r\201 (in %)");
  ytics(0,100,20,2);

  setwind(1);
    title("\216b = 60%");
    xy(x1,y1[.,1 2]);

  setwind(2);
    title("\216b = 90%");
    xy(x1,y1[.,3 4]);

  xlabel("\216b (in %)");
  xtics(0,100,25,2);

  setwind(3);
    title("\216\202r\201 = 0%");
    xy(x2,y2[.,1 2]);

  setwind(4);
    title("\216\202r\201 = 90%");
    _plegstr = "\202s\201]2[ = \202s\201]1[\000\202s\201]2[ = 3x\202s\201]1[";
    _plegctl = {2 6 6 2};
    xy(x2,y2[.,3 4]);

    graphprt("-c=2 -cf=chap2-2-2-1.eps");

  endwind;


