/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 107 (Figure 2.5)
**
*/

new;
library rpb,pgraph;

cls;

rndseed 123;

/*
**> Constant Correlation Matrix
**
*/

sigma = 0.20;

scalar_rho = seqa(0,0.01,101);
nRho = rows(scalar_rho);

let rb = 0.10 0.70 0.90;
nRB = rows(rb);

x = zeros(nRho,nRB);

n = 5;
i = 1;
do until i > nRho;
  rho = diagrv(scalar_rho[i] * ones(n,n),1.0);
  cov = rho .* sigma .* sigma';

  x0 = ones(n,1)/n;

  j = 1;
  do until j > nRB;
    b = zeros(n,1);
    b[1] = rb[j];
    b[2:n] = (1-b[1])/(n-1) * ones(n-1,1);
    {sol,vol,mr,tr,rc} = Compute_RB_Portfolio(x0,cov,b);
    x[i,j] = sol[1];
    j = j + 1;
  endo;

  i = i + 1;
endo;

x1 = scalar_rho;
y1 = x;

n = 30;
i = 1;
do until i > nRho;
  rho = diagrv(scalar_rho[i] * ones(n,n),1.0);
  cov = rho .* sigma .* sigma';

  x0 = ones(n,1)/n;

  j = 1;
  do until j > nRB;
    b = zeros(n,1);
    b[1] = rb[j];
    b[2:n] = (1-b[1])/(n-1) * ones(n-1,1);
    {sol,vol,mr,tr,rc} = Compute_RB_Portfolio(x0,cov,b);
    x[i,j] = sol[1];
    j = j + 1;
  endo;

  cls;

  i = i + 1;
endo;

x2 = scalar_rho;
y2 = x;

n = seqa(1,1,50);
nAssets = rows(n);
scalar_rho = 0;
x = miss(zeros(nAssets,nRB),0);

i = 1;
do until i > nAssets;

  if i == 1;
    i = i + 1;
    continue;
  endif;

  rho = diagrv(scalar_rho * ones(n[i],n[i]),1.0);
  cov = rho .* sigma .* sigma';

  x0 = ones(n[i],1)/n[i];

  j = 1;
  do until j > nRB;
    b = zeros(n[i],1);
    b[1] = rb[j];
    b[2:n[i]] = (1-b[1])/(n[i]-1) * ones(n[i]-1,1);
    sol = Compute_RB_CC(n[i],sigma,scalar_rho,b);
    x[i,j] = sol[1];
    j = j + 1;
  endo;

  i = i + 1;
endo;

x3 = n;
y3 = x;

n = seqa(1,1,50);
nAssets = rows(n);
scalar_rho = 0.20;
x = miss(zeros(nAssets,nRB),0);

i = 1;
do until i > nAssets;

  if i == 1;
    i = i + 1;
    continue;
  endif;

  rho = diagrv(scalar_rho * ones(n[i],n[i]),1.0);
  cov = rho .* sigma .* sigma';

  x0 = ones(n[i],1)/n[i];

  j = 1;
  do until j > nRB;
    b = zeros(n[i],1);
    b[1] = rb[j];
    b[2:n[i]] = (1-b[1])/(n[i]-1) * ones(n[i]-1,1);
    {sol,vol,mr,tr,rc} = Compute_RB_Portfolio(x0,cov,b);
    x[i,j] = sol[1];
    j = j + 1;
  endo;

  cls;

  i = i + 1;
endo;

x4 = n;
y4 = x;


graphset;
  begwind;
  window(2,2,0);

  fonts("simplex simgrma");
  _pdate = ""; _pframe = 0; _pnum = 2;
  _pltype = 6|3|1; _pcolor = 1|2|4; _plwidth = 10;
  _paxht = 0.25;  _ptitlht = 0.275;  _pnumht = 0.20;

  xtics(0,100,50,2);
  xlabel("\216\202r\201 (in %)");
  ytics(0,100,20,2);

  setwind(1);
    title("\216n = 5");
    xy(100*x1,100*y1);

  setwind(2);
    title("\216n = 30");
    xy(100*x2,100*y2);

  xtics(0,30,10,2);
  xlabel("\216n");
  ytics(0,100,20,2);

  setwind(3);
    title("\216\202r\201 = 0%");
    xy(x3,100*y3);

  setwind(4);
    title("\216\202r\201 = 20%");
    _plegstr = "b]1[ = 10%\000b]1[ = 70%\000b]1[ = 90%";
    _plegctl = {2 5.5 6 1.6};
    xy(x4,100*y4);

    graphprt("-c=2 -cf=chap2-2-2-3.eps");

  endwind;
