/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 210 (Figure 4.7)
**
*/

new;
library rpb,pgraph;

cls;

_Bond_Continuous_Time = 1;
_bond_window = 2;

loadm maturity = data\zc-tau;

datFileName = "data\\zc-ir-2009.dat";
data = loadd(datFileName);
data = selif(data,(data[.,1] .>= 20100630) .and (data[.,1] .<= 20120630));
Dates = data[.,1];
nDates = rows(Dates);
R = data[.,2:cols(data)]/100;
tau = seqa(1,1,5);
indx = indnv(tau,Maturity);
R = R[.,indx];

rndseed 123456;
t = seqa(0,1/260,nDates);
lambda = simulate_GBM(1,0.50,0.40,t,1)/100;

Rt = R[rows(R),.]';

Nominal = 1e6;
Coupon = 5;
Notional = 100;
Maturity = 5;
Ct = Coupon * ones(Maturity,1);
Ct[Maturity] = Ct[Maturity] + Notional;
t = seqa(1,1,Maturity);

P0 = Bond_Price(t,Ct,Rt);
nB = Nominal/P0;
nB = trunc(nB);

dR = R - lag1(R);
{gVaR1,gVaR2,hVaR1,hVaR2,hVaR3,ES,results} = Bond_VaR(t,Ct,Rt,dR,0.99,1/260);
print ftosa(nB*(gVaR1~gVaR2~hVaR1~hVaR2~hVaR3~ES),"%lf",5,1);
print "--------------------------------------------------------------------------------------------------------";

P0_ND = P0;
nB_ND = nB;

lambda_t = lambda[rows(lambda)];
RR_t = 0.50;
P0 = Risky_Bond_Price_EDT(t,Ct,Rt,Notional,Maturity,RR_t,lambda_t);
nB = Nominal/P0;
nB = trunc(nB);

dLambda = Lambda ./ lag1(Lambda) - 1;

R = Rt + dR';
lambda = lambda_t .* (1 + dLambda);
t = t - 1/260;
Maturity = Maturity - 1/260;

nScenario = 250;
P1_ND = zeros(nDates,1);
P1 = zeros(nDates,1);
P2 = zeros(nDates,nScenario);
P3 = zeros(nDates,nScenario);

i = 1;
do until i > nDates;
  P1_ND[i] = Risky_Bond_Price_EDT(t,Ct,R[.,i],Notional,Maturity,0,0);
  P1[i] = Risky_Bond_Price_EDT(t,Ct,R[.,i],Notional,Maturity,RR_t,lambda[i]);
  j = 1;
  do until j > nScenario;
    RR = 0.45 + rndu(1,1) * 0.10;
    P2[i,j] = Risky_Bond_Price_EDT(t,Ct,R[.,i],Notional,Maturity,RR,lambda[i]);
    RR = 0.40 + rndu(1,1) * 0.20;
    P3[i,j] = Risky_Bond_Price_EDT(t,Ct,R[.,i],Notional,Maturity,RR,lambda[i]);
    j = j + 1;
  endo;

  i = i + 1;
endo;

P1_ND = packr(P1_ND);
P1 = packr(P1);
P2 = packr(vec(P2));
P3 = packr(vec(P3));

Loss0 = nB_ND * (P0_ND - P1_ND);
Loss1 = nB * (P0 - P1);
Loss2 = nB * (P0 - P2);
Loss3 = nB * (P0 - P3);

alpha = 0.99;
hVaR0 = quantile(Loss0,alpha);
hVaR1 = quantile(Loss1,alpha);
hVaR2 = quantile(Loss2,alpha);
hVaR3 = quantile(Loss3,alpha);

print;
print ftos(hVaR0,"Historical VaR without default risk          = %lf",5,2);
print ftos(hVaR1,"Historical VaR with constant recovery rate   = %lf",5,2);
print ftos(hVaR2,"Historical VaR with stochastic recovery rate = %lf",5,2);
print ftos(hVaR3,"Historical VaR with stochastic recovery rate = %lf",5,2);

Loss0 = Loss0/1e3;
Loss1 = Loss1/1e3;
Loss2 = Loss2/1e3;

x = seqa(-30,60/250,251);
{d0,F0} = regKernelDensity(Loss0,x);
{d1,F1} = regKernelDensity(Loss1,x);
{d2,F2} = regKernelDensity(Loss2,x);

graphset;
  _pdate = ""; _pnum = 2; _pframe = 0; _pysci = 1;
  _pltype = 6|1|3; _plwidth = 10; _pcolor = 1|2|4;
  xtics(-20e3,20e3,5e3,2);
  xlabel("\216Loss");
  _plegstr = "Without default risk\000Constant recovery rate\000Stochastic recovery rate";
  _plegctl = {2 5 5.5 5};
  graphprt("-c=2 -cf=chap4-2-3-6.eps");
  xy(1e3*x,d0~d1~d2);




