/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 259 (Table 5.5)
**
*/

new;
library rpb,pgraph;

cls;

datPathName = "Data\\";
datFileName = datPathName $+ "djcs-hf";

resPathName = "Results\\DJCS-3F";

Data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

let string Selected_Names[10,1] = CA SB EM EMN ED FIA GM LSE MF MS;

indx = indsav(Selected_Names,vnames);
nAssets = rows(indx);

Dates = Data[.,1];
nDates = rows(Dates);
t = seqa(2004 + 9/12,1/12,nDates);
Index_NAV = Data[.,2];
SubIndex_NAV = Data[.,indx];

R_SubIndex = SubIndex_NAV ./ lag1(SubIndex_NAV) - 1.0;
nLags = 24;

let x0 = 3 1 6 4 22 5 14 24 6 13;
x0 = x0 / sumc(x0);
index_weights = x0' .* ones(nDates,1);

stats = zeros(10,8);
alpha = 0.95;

i = 1;
do until i > 10;
  Rdt = R_SubIndex[.,i];
  Rdt = packr(Rdt);
  stats[i,1] = 100*sqrt(12) * stdc(Rdt);
  sk = Estimate_Skewness(Rdt);
  kt = Estimate_Kurtosis(Rdt);
  stats[i,2] = sk;
  stats[i,3] = kt - 3;
  {MDD,beg_MDD,end_MDD} = Estimate_Maximum_Drawdown(SubIndex_NAV[.,i],1);
  stats[i,4] = 100*MDD;

  vol = stdc(Rdt);
  mu = meanc(Rdt);
  VaR = -mu + cdfni(alpha) * vol;
  stats[i,5] = 100 * VaR;
  stats[i,6] = -100 * quantile(Rdt,1-alpha);

  c = pdfn(cdfni(alpha))/(1-alpha);
  ES = -mu + c * vol;
  stats[i,7] = 100*ES;

  VaR = quantile(Rdt,1-alpha);
  Loss = selif(Rdt,Rdt .<= VaR);
  stats[i,8] = -100 * meanc(Loss);

  i = i + 1;
endo;

print ftosa(stats,"%lf",3,1);

