/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 258 & 262 (Simulates ERC allocations)
**
*/

new;
library rpb,pgraph;

cls;

#include chap5-2-2-3.inc

datPathName = "Data\\";
datFileName = datPathName $+ "djcs-hf";

resPathName = "Results\\DJCS-3F";

Data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

let string Selected_Names[10,1] = CA SB EM EMN ED FIA GM LSE MF MS;

indx = indsav(Selected_Names,vnames);
nAssets = rows(indx);

Dates = Data[.,1];
nDates = rows(Dates);
t = seqa(2004 + 9/12,1/12,nDates);
Index_NAV = Data[.,2];
SubIndex_NAV = Data[.,indx];

R_SubIndex = SubIndex_NAV ./ lag1(SubIndex_NAV) - 1.0;
nLags = 24;

let x0 = 3 1 6 4 22 5 14 24 6 13;
x0 = x0 / sumc(x0);
index_weights = x0' .* ones(nDates,1);
x_star = x0;

nIters = 7;
Asset_Weights = arrayinit(nIters|nDates|nAssets,error(0));
Asset_RC = arrayinit(nIters|nDates|nAssets,error(0));

let indx_Factors = 1 2 3;
let b_Factors = 1 1 1;
b_Factors = b_Factors ./ sumc(b_Factors);

nFactors = rows(indx_Factors);
Factor_RC = arrayinit(nIters|nDates|nFactors+1,error(0));
FRC_Index = arrayinit(nIters|nDates|7,error(0));

MaxIters = 50;
_co_MaxIters = MaxIters;
_sqp_MaxIters = MaxIters;
_use_co = 1;

test_fmin = miss(zeros(nDates,nIters),0);
test_retcode = miss(zeros(nDates,nIters),0);

alpha = 0.80;

i = 1;
do until i > nDates;

  cls;
  
  print /flush ftos(i,"Iteration = %lf",1,0);
  
  if i <= nLags + 1;
    i = i + 1;
    continue;
  endif;

  if i == 80;
    R = R_SubIndex[i-nLags-1:i,.];
  else;
    R = R_SubIndex[i-nLags:i,.];
  endif;

  cov = vcx(R);
  {va,ve} = eighv(cov);
  va = rev(va);
  ve = ve[.,seqa(cols(ve),-1,cols(ve))];
  va = va[indx_factors]; ve = ve[.,indx_factors];
  Omega = diagrv(eye(nFactors),va);
  A = ve;
  D = cov - A*Omega*A';
  
  oldtrap = trapchk(1);
  trap 1,1;
  P = pinv(A);
  trap oldtrap,1;
  if scalerr(P) or (maxc(maxc(P)) == __INFp);
    str = itos_rpb("error in pinv at iteration ",i);
    errorlog str;
    err = 1;
  else;
    err = 0;
  endif;
  
  iter = 1;
  do until iter > nIters;
  
  if err == 1;
    Asset_Weights[iter,i,.] = Asset_Weights[iter,i-1,.];
    Asset_RC[iter,i,.] = Asset_RC[iter,i-1,.];
    Factor_RC[iter,i,.] = Factor_RC[iter,i-1,.];
    FRC_Index[iter,i,.] = FRC_Index[iter,i-1,.];
    i = i + 1;
    continue;
  endif;

  x0 = Starting_Value(cov,iter,i);
    
  x0 = x0/sumc(x0);
  
  if iter == 1;         // Asset-weighted

    x = x_star;   
    {risk_x,mr_x,rc_x,prc_x} = Compute_RC_NG(x,R,alpha,1);  
    _rp_fmin = 0; 
    _rp_retcode = 0;

  elseif iter == 2;     // ERC Volatility

    {x,risk_x,mr_x,rc_x,prc_x} = Compute_RB_NG(x0,R,0,alpha,1);

  elseif iter == 3;     // ERC Expected Shortfall

    {x,risk_x,mr_x,rc_x,prc_x} = Compute_RB_NG(x0,R,0,alpha,2);

  elseif iter == 4;     // ERC VaR Cornish-Fisher

    {x,risk_x,mr_x,rc_x,prc_x} = Compute_RB_NG(x0,R,0,alpha,3);

  elseif iter == 5;     // PCA Factor Volatility

    {x,risk_x,mr_x,rc_x,prc_x} = Compute_RB_Factor_NG(x0,R,A,Omega,D,b_Factors,indx_Factors,alpha,1);

  elseif iter == 6;     // PCA Factor Expected Shortfall

    {x,risk_x,mr_x,rc_x,prc_x} = Compute_RB_Factor_NG(x0,R,A,Omega,D,b_Factors,indx_Factors,alpha,2);

  elseif iter == 7;    // PCA Factor VaR Cornish-Fisher

    {x,risk_x,mr_x,rc_x,prc_x} = Compute_RB_Factor_NG(x0,R,A,Omega,D,b_Factors,indx_Factors,alpha,3);

  endif;

  test_fmin[i,iter] = _rpb_fmin;
  test_retcode[i,iter] = _rpb_retcode;
  Asset_Weights[iter,i,.] = x';
  Asset_RC[iter,i,.] = prc_x';
  
  {y,risk_y,mr_y,rc_y,prc_y} = Compute_RC_Factor_NG(x,risk_x,mr_x,A,Omega,D);

  Factor_RC[iter,i,.] = prc_y[1:nFactors]' ~ sumc(prc_y[1+nFactors:nAssets]);
  
  p = prc_y;
  p = selif(p , p .> 0);
  {H,H_star,N_eff} = Herfindahl_Index(p);
  {Gini,Gini_star} = Gini_Index(p);
  {SE,SE_star}     = Entropy_Index(p);
  FRC_Index[iter,i,.] = H~H_star~N_eff~Gini~Gini_star~SE~SE_star;
  
  iter = iter + 1;
  endo;
  
  i = i + 1;
endo;

save path = ^resPathName Asset_Weights;
save path = ^resPathName Asset_RC;
save path = ^resPathName Factor_RC;
save path = ^resPathName FRC_Index;
save path = ^resPathName test_retcode;
save path = ^resPathName test_fmin;

