/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 275 & 276 (Table 6.1 & Figure 6.4)
**
*/

new;
library rpb,pgraph;

cls;

declare matrix rho1,rho2,rho3;

PathName = "data\\";
datFileName = PathName $+ "msci-wgbi-djubs-fed"; 
data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

Begin_Date = 19981231;
End_Date = 20111230;

data = selif(data,(data[.,1] .>= Begin_Date) .and (data[.,1] .<= End_Date));
data = Fill_Miss(data,4);

Dates = data[.,1];
nDates = rows(Dates);

Data = Data[.,2 3];
Data = 100 * Data ./ Data[1,.];

{cov,vol,cor} = Rolling_Covariance(Dates,Data,260,1);

let w1 = 0.20 0.80;
let w2 = 0.50 0.50;
let w3 = 0.80 0.20;

pi1 = zeros(nDates,2);
pi2 = zeros(nDates,2);
pi3 = zeros(nDates,2);

SR = 0.25;

t = 1;
do until t > nDates;

  Sigma = arraytomat(cov[t,.,.]);

  {vol,mr,rc,prc} = Compute_Risk_Contribution(w1,Sigma);
  pi1[t,.] = SR * mr';

  {vol,mr,rc,prc} = Compute_Risk_Contribution(w2,Sigma);
  pi2[t,.] = SR * mr';

  {vol,mr,rc,prc} = Compute_Risk_Contribution(w3,Sigma);
  pi3[t,.] = SR * mr';

  t = t + 1;
endo;

t = selif(seqa(1,1,nDates),Dates .>= 20000101);
pi1 = pi1[t,.];
pi2 = pi2[t,.];
pi3 = pi3[t,.];
nDates = rows(t);
t = seqa(1,1,nDates);

step = 5;
indx_t = seqa(1,step,nDates/step)|nDates;

graphset;
  window3u;

  fonts("simplex simgrma");

  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6|1|3; _plwidth = 10; _pcolor = 1|2;
  _pnumht = 0.20; _paxht = 0.23; _ptitlht = 0.25; _pcross = 1;

  xtics(1,nDates,261*2,2);
  let xlab = "2000" "02" "04" "06" "08" "10" "12";
  asclabel(xlab,0);
  ytics(-1,7,1,2);

  setwind(1);
    title("\216Defensive");
    ylabel("\216\202p\201 (in %)");
    xy(t[indx_t],100*pi1[indx_t,.]);

  ytics(-2,10,2,2);

  setwind(2);
    title("\216Balanced");
    xy(t[indx_t],100*pi2[indx_t,.]);

  setwind(3);
    title("\216Dynamic");
    _plegstr = "Equity\000Bond";
    _plegctl = {2 6 5 9};
    xy(t[indx_t],100*pi3[indx_t,.]);

    graphprt("-c=2 -cf=chap6-1-2-1.eps");

  endwind;

stats = zeros(2,6);

p_lag = 1;

i = 1;
do until i > 3;
  pi_ = varget(itos_rpb("pi",i));
  stats[.,i] = meanc(pi_);
  stats[.,3+i] = sqrt(260) * stdc(pi_);

  d = pi_ - lagn(pi_,p_lag);
  d = packr(d);
  rho = corrx(d);
  call varput(rho,itos_rpb("rho",i));
  i = i + 1;
endo;

indx = 2;
pi_ = pi1[.,indx]~pi2[.,indx]~pi3[.,indx];
d = pi_ - lagn(pi_,p_lag);
d = packr(d);
rho_Bond = corrx(d);

p_lag = 1;

indx = 1;
pi_ = pi1[.,indx]~pi2[.,indx]~pi3[.,indx];
d = pi_ - lagn(pi_,p_lag);
d = packr(d);
rho_Equity = corrx(d);

indx = 2;
pi_ = pi1[.,indx]~pi2[.,indx]~pi3[.,indx];
d = pi_ - lagn(pi_,p_lag);
d = packr(d);
rho_Bond = corrx(d);

print ftosa(100*stats,"%lf",3,2);
print "---------------------------------------------------------------";
print ftosa(100*rho_Equity,"%lf",3,0);
print "---------------------------------------------------------------";
print ftosa(100*rho_Bond,"%lf",3,0);
print "---------------------------------------------------------------";

print ftosa(100*rho1,"%lf",3,0);
print "---------------------------------------------------------------";
print ftosa(100*rho2,"%lf",3,0);
print "---------------------------------------------------------------";
print ftosa(100*rho3,"%lf",3,0);
print "---------------------------------------------------------------";

