/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Pages 279 & 280 (Table 6.2 & Figure 6.7)
**
*/

new;
library rpb,pgraph;

cls;

PathName = "data\\";
datFileName = PathName $+ "msci-wgbi-djubs-fed"; 
data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

Begin_Date = 19981231;
End_Date = 20111230;

data = selif(data,(data[.,1] .>= Begin_Date) .and (data[.,1] .<= End_Date));
data = Fill_Miss(data,4);

Dates = data[.,1];
nDates = rows(Dates);

Months = trunc(Dates/100);
RD_Monthly = Months ./= lag1(Months);
RD_Monthly[1] = 1;

Basket = Data[.,2 3];
Basket = 100 * Basket ./ Basket[1,.];
Libor = Data[.,5]/100;

{cov,vol,cor} = Rolling_Covariance(Dates,Basket,260,1);

let w1 = 0.20 0.80;
let w2 = 0.50 0.50;
let w3 = 0.80 0.20;

weights1 = miss(zeros(nDates,2),0);
weights2 = miss(zeros(nDates,2),0);
weights3 = miss(zeros(nDates,2),0);
weights4 = miss(zeros(nDates,2),0);

rc1 = miss(zeros(nDates,2),0);
rc2 = miss(zeros(nDates,2),0);
rc3 = miss(zeros(nDates,2),0);
rc4 = miss(zeros(nDates,2),0);

all_vol = zeros(nDates,4);

w4 = ones(2,1)/2;

t = 1;
do until t > nDates;

  Sigma = arraytomat(cov[_max_(t,260),.,.]);

  if RD_Monthly[t] == 1;

    w = w1;
    {vol,mr,rc,prc} = Compute_Risk_Contribution(w,Sigma[1:2,1:2]);
    weights1[t,.] = w';
    rc1[t,.] = prc';

    w = w2;
    {vol,mr,rc,prc} = Compute_Risk_Contribution(w,Sigma[1:2,1:2]);
    weights2[t,.] = w';
    rc2[t,.] = prc';

    w = w3;
    {vol,mr,rc,prc} = Compute_Risk_Contribution(w,Sigma[1:2,1:2]);
    weights3[t,.] = w';
    rc3[t,.] = prc';

    {w,vol,mr,rc,prc} = Compute_ERC_Portfolio(w4,Sigma[1:2,1:2]);
    weights4[t,.] = w';
    rc4[t,.] = prc';

  else;

    w = weights1[t-1,.]';
    {vol,mr,rc,prc} = Compute_Risk_Contribution(w,Sigma[1:2,1:2]);
    weights1[t,.] = w';
    rc1[t,.] = prc';
    all_vol[t,1] = vol;

    w = weights2[t-1,.]';
    {vol,mr,rc,prc} = Compute_Risk_Contribution(w,Sigma[1:2,1:2]);
    weights2[t,.] = w';
    rc2[t,.] = prc';
    all_vol[t,2] = vol;

    w = weights3[t-1,.]';
    {vol,mr,rc,prc} = Compute_Risk_Contribution(w,Sigma[1:2,1:2]);
    weights3[t,.] = w';
    rc3[t,.] = prc';
    all_vol[t,3] = vol;

    w = weights4[t-1,.]';
    {vol,mr,rc,prc} = Compute_Risk_Contribution(w,Sigma[1:2,1:2]);
    weights4[t,.] = w';
    rc4[t,.] = prc';
    all_vol[t,4] = vol;

  endif;

  t = t + 1;
endo;

Backtests = zeros(nDates,10);
Backtests[.,1] = Simulate_Monthly_Backtest(Dates,Weights1,Basket[.,1:2],0,0);
Backtests[.,2] = Simulate_Monthly_Backtest(Dates,Weights2,Basket[.,1:2],0,0);
Backtests[.,3] = Simulate_Monthly_Backtest(Dates,Weights3,Basket[.,1:2],0,0);
Backtests[.,4] = Simulate_Monthly_Backtest(Dates,Weights4,Basket[.,1:2],0,0);

Weights5 = meanc(Weights4)' .* ones(nDates,1);
Backtests[.,5] = Simulate_Monthly_Backtest(Dates,Weights5,Basket[.,1:2],0,0);

Backtests[.,10] = Capitalized_Libor(Dates,Libor);

/*
**> Leverage effect
**
*/

R_RP = Backtests[.,4]./lag1(Backtests[.,4]) - 1.0;
R_Libor = lag1(Libor)/260;
R_RP[1] = 0.0;
lambda = 0.96;
sigma_t = zeros(nDates,1);
sigma_t[1] = sqrt(260) * stdc(R_RP);

i = 2;
do until i > nDates;
  dt = etdays2(dates[i-1],dates[i])/365;
  sigma_t[i] = sqrt(lambda * sigma_t[i-1]^2 + (1-lambda) * R_RP[i]^2 / dt);
  i = i + 1;
endo;

proc (1) = VolTarget_Exposure(vol,vol_target,min_lev,max_lev);
  local lev;

  lev = vol_target ./ vol;
  lev = _min_(_max_(lev,min_lev),max_lev);

  retp(lev);
endp;

let vol_target[1,3] = 0.06 0.08 0.10;
w = VolTarget_Exposure(sigma_t,vol_target,0.0,2.0);
w = lag1(w);
w[1,.] = w[2,.];
R_star = w .* R_RP + (1 - w) .* R_Libor;
R_star = R_star~R_Libor;
R_star[1,.] = zeros(1,4);
Backtests[.,6:9] = 100 * cumprodc(1 + R_star);

t_Daily = selif(seqa(1,1,nDates), Dates .>= 20000101);
Dates = Dates[t_Daily];
nDates = rows(Dates);
Backtests = Backtests[t_Daily,.];
Backtests = 100 * Backtests./Backtests[1,.];
Weights4 = Weights4[t_Daily,.];
RD_Monthly = RD_Monthly[t_Daily];

indx_t = selif(seqa(1,1,nDates),RD_Monthly);
t = seqa(1,1,rows(indx_t));

graphset;
  window3u;

    _pdate = ""; _pnum = 2; _pframe = 0;
    _pltype = 6; _plwidth = 10;
    _paxht = 0.20; _pnumht = 0.20; _ptitlht = 0.25;

    xtics(1,rows(t),12*2,2);
    ytics(0,100,20,2);

    _pbarwid = 1;
    let xlab = "2000" "02" "04" "06" "08" "10" "12";
    asclabel(xlab,0);

    _pcolor = 1|2;
    setwind(1);
      title("\216Weights of the risk parity portfolio (in %)");
      w = weights4[.,1]~sumr(weights4);

      _pmsgstr = "Equity\00Bond";
      _pmsgctl = 70~15-0.35~0.25~0~1~1~10 |
                 70~70-0.35~0.25~0~1~2~10 ;

       xy(t,100*w[indx_t,.]);

    _pltype = 6|1|3; _pcolor = 1|2|3; _pmsgctl = 0;

    setwind(2); 
       title("\216Performance of diversified funds");
       ytics(50,225,25,2);
      _plegstr = "Defensive\000Balanced\000Dynamic";
      _plegctl = {2 6 1.25 4};
      xy(t,backtests[indx_t,1:3]);

    _pcolor = 4|6|12;
    setwind(3); 
      title("\216Performance of risk parity funds");
      _plegstr = "Risk parity\000Static\000Leveraged risk parity";
      _plegctl = {2 6 4.25 1};
      xy(t,backtests[indx_t,4 5 8]);

      graphprt("-c=2 -cf=chap6-1-3-1.eps");

  endwind;

cls;


stats = zeros(6,10);

iter = 1;
do until iter > 10;
  report = Reporting_Backtest(Dates[.,1],Backtests[.,iter],0,Backtests[.,10],0,0);
  stats[1,iter] = vread(Report,"mu");
  stats[2,iter] = vread(Report,"sigma");
  stats[3,iter] = vread(Report,"SR");
  stats[4,iter] = vread(Report,"MDD");
  stats[5,iter] = vread(Report,"sk");
  stats[6,iter] = vread(Report,"kt")-3.0;
  iter = iter + 1;
endo;

stats[1:2 4,.] = stats[1:2 4,.] * 100;
print ftosa(stats[.,1:5 8]',"%lf",3,2);

