/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 283 (Figure 6.8)
**
*/

new;
library rpb,pgraph;

cls;

n = 15;
let m_j[5,4] = 1.00 0.80 0.70 0.50
               1.00 0.80 0.70 0.50
               1.00 1.00 0.70 0.50
               1.00 1.00 0.70 0.50
               1.00 1.00 1.00 1.00;
m = rows(m_j);
nIters = cols(m_j);

r = 0.02;

sharpe = 0.5;
sigma = seqa(0.03,0.01,n);
mu = r + sharpe * sigma;
rho = diagrv(0.5*ones(n,n),1.0);
cov = rho .* sigma .* sigma';

A = ones(1,n); B = 1;
gamma_ = seqa(0,1/4000,10000);
Bounds = (-1e300~1e300) .* ones(n,1);

{x0,mu0,sigma0,gamma_,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,gamma_,0);  

SR = (mu0 - r) ./ sigma0;
indx = maxindc(SR);
x_bar = x0[.,indx]; 
mu_x_bar = mu0[indx]; 
sigma_x_bar = sigma0[indx]; 
gamma_bar = gamma_[indx];

e = eye(n);
beta_bar = (e'*cov*x_bar)/(x_bar'*cov*x_bar);
mu_beta_bar = beta_bar .* (mu_x_bar - r);
print ftosa(seqa(1,1,n)~100*(mu-r)~beta_bar~100*mu_beta_bar,"%lf",3,2);
print "------------------------------------------------------------------------------------------------------";

C = ones(1,n);
gamma_tilde = gamma_bar;
x_tilde_j = zeros(n,m);
lambda_tilde = zeros(m,1);
Beta_results = zeros(n,nIters);
alpha_results = zeros(n,nIters);

iter = 1;
do until iter > nIters;

  j = 1;
  do until j > m;
    D = m_j[j,iter];
    {x_tilde_j[.,j],mu_tilde,sigma_tilde,gamma_,retcode} = Compute_MVO_Portfolio(mu,cov,0,0,-C,-D,Bounds,gamma_tilde,0);  
    {x,u1,u2,u3,u4,retcode} = QProg(_rpb_sv,1e-2*_rpb_Q,_rpb_R,_rpb_A,
                                    _rpb_B,_rpb_C,_rpb_D,_rpb_bounds);
    lambda_tilde[j] = u2;
    j = j + 1;
  endo;

  x_tilde = meanc(x_tilde_j');
  x_tilde = x_tilde/sumc(x_tilde);
  mu_x_tilde = x_tilde'mu;
  psi_ = sumc(lambda_tilde .* m_j[.,iter]);

  beta_tilde = (e'*cov*x_tilde)/(x_tilde'*cov*x_tilde);
  mu_beta_tilde = beta_tilde .* (mu_x_tilde - r);
  alpha_tilde = psi_ * (1 - beta_tilde);

  scale_alpha = (mu-r-mu_beta_tilde)/alpha_tilde;
  alpha_tilde = scale_alpha * alpha_tilde;

  print ftosa(seqa(1,1,n)~100*(mu-r)~beta_tilde~100*alpha_tilde~100*mu_beta_tilde~100*(mu_beta_tilde+alpha_tilde),"%lf",3,2);
  print "------------------------------------------------------------------------------------------------------";
  
  Beta_results[.,iter] = beta_tilde;
  alpha_results[.,iter] = alpha_tilde;

  iter = iter + 1;
endo;

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0; _pcross = 1;
  _plctrl = -1; _pcolor = 1|4|2|3; _pltype = 6; _pstype = 8|11|12|9;
  xlabel("\216\202b\201]i[\201");
  ylabel("\216\202a\201]i[\201 (in bps)");
  xtics(0,2.0,0.5,2);
  _plegstr = "Set #1\000Set #2\000Set #3\000Set #4\000Set #5";
  _plegctl = {2 5 6 4};
  graphprt("-c=2 -cf=chap6-1-4-1.eps");
  xy(beta_results,1e4*alpha_results);




