/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 283 (Figure 6.9)
**
*/

new;
library rpb,pgraph;

cls;

let mu =    0.05 0.06 0.07 0.06;
let sigma = 0.06 0.11 0.16 0.20;
n = rows(Sigma);
let rho =   1.00 
            0.10   1.00
            0.40   0.70   1.00
            0.50   0.40   0.80 1.00;
rho = xpnd(rho);
call invpd(rho);

cov = rho .* sigma .* sigma';
A = ones(1,n); B = 1;

Bounds = (-10~10) .* ones(n,1);
gamma_ = seqa(-2,1/100,800);
{x0,mu0,sigma0,gamma0,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,gamma_,0);  

r = 0.03;

SR = (mu0-r)./sigma0;
indx = maxindc(SR);
mu1 = mu0[indx]; sigma1 = sigma0[indx]; SR1 = SR[indx];
sigma2 = 0.40; mu2 = r + SR1 * sigma2;

{x3,mu3,sigma3,gamma3,retcode} = Compute_MVO_Portfolio(mu,cov,A,B,0,0,Bounds,0.10|0.13|0.20,2);  
e3 = ones(cols(x3),1);

sigma4 = 0.15;
mu4 = 0.065;

sigma5 = sigma1 + 0.01;
mu5 = mu1 + 0.0005;
SR = (mu5-r)./sigma5;
sigma6 = 0.30; mu6 = r + SR * sigma6;
sigma7 = sigma4; mu7 = r + SR * sigma7;

graphset;
  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0;
  _pltype = 6; _plwidth = 10; _pcolor = 1;
  xtics(0,25,5,2);
  ytics(0,12,2,0);
  xlabel("\216Volatility (in %)");
  ylabel("\216Expected return (in %)");
  _psym = 0~100*r~11~5~5~1~8 |
          100*sigma1~100*mu1~11~5~5~1~8 |
          100*sigma3~100*mu3~e3.*(11~5~8~1~8) |
          100*sigma4~100*mu4~11~5~2~1~8 |
          100*sigma5~100*mu5~11~5~3~1~8 |
          100*sigma7~100*mu7~11~5~3~1~8 ;

  _pline = 1~6~0~100*r~100*sigma2~100*mu2~0~5~10 |
           1~3~0~100*r~100*sigma6~100*mu6~0~3~10 ;
 
  _pmsgstr = "Risk-free asset\000Tangency portfolio\000A\000B\000C\000Market portfolio\000Risk parity\000Leveraged risk parity";
  _pmsgctl =   0.5~100*r-0.50~0.14~0~1~5~7 |
               100*sigma1-6.9~100*mu1+0.10~0.14~0~1~5~7 |
               100*(sigma3[1]+0.004)~100*(mu3[1]-0.003)~0.14~0~1~8~7 |
               100*(sigma3[2]+0.004)~100*(mu3[2]-0.003)~0.14~0~1~8~7 |
               100*(sigma3[3]+0.004)~100*(mu3[3]-0.003)~0.14~0~1~8~7 |
               100*sigma4+0.5~100*mu4-0.15~0.14~0~1~2~7 |
               100*sigma5+0.5~100*mu5-0.15~0.14~0~1~3~7 |
               100*sigma7+0.5~100*mu7-0.15~0.14~0~1~3~7 ;  
  graphprt("-c=2 -cf=chap6-1-4-2.eps");
  xy(100*sigma0,100*mu0);


