/*
** Thierry Roncalli
** Introduction to Risk Parity & Budgeting
**
**> Page 293 (Figure 6.14)
**
*/

new;
library rpb,pgraph;

cls;

mtd = 0;

datPathName = "Data\\";
datFileName = datPathName $+ "rfp-ffc";

Begin_Date = 199201;
End_Date = 201206;

Begin_Date = 100 * Begin_Date + 15;
End_Date = 100 * End_Date + 15;

data = loadd(datFileName);
vnames = "" $+ getname(datFileName);

data = selif(data,(data[.,1] .>= Begin_Date) .and (data[.,1] .<= End_Date));

data = packr(data);
Dates = data[.,1];
nDates = rows(Dates);

let string Selected_Names[4,1] = "MKT" "SMB" "HML" "MOM";
indx_Factors = indsav(Selected_Names,vnames);
R_Factors = Data[.,indx_Factors];

let string Selected_Names[5,1] = "SP100" "EW" "MV" "MDP" "ERC";
indx_Indexes = indsav(Selected_Names,vnames);
nIndexes = rows(indx_Indexes);

R_Indexes = Data[.,indx_Indexes];

nLags = 0;
if nLags == 0;
  nLags = nDates;
endif;

t = nDates;
all_rc = zeros(4,4);
all_prc = zeros(4,4);

R = R_Indexes[t-nLags+1:t,.];
F = R_Factors[t-nLags+1:t,.];

A = R/F;

if mtd == 1;
  n = cols(R);
  m = cols(F);
  Sigma = 12*vcx(R);
  Omega = 12*vcx(F);
  A = A';
  D = Sigma - A*Omega*A';
  D = diagrv(eye(n),diag(D));
  Sigma = A*Omega*A' + D;
else;
  A = A';
endif;

print ftosa(A,"%lf",3,2);

iter = 2;
do until iter > nIndexes;

  if mtd == 1;
    x = zeros(n,1); 
    x[1] = -1.0;
    x[iter] = 1.0;
    {vol,y,mr,rc,prc} = Compute_RC_Factor(x,Sigma,A,Omega,D);
  else;
    y = R[.,iter] - R[.,1];
    n = cols(y);
    x = F;
    m = cols(x);
  
    Sigma = 12*vcx(y);
    Omega = 12*vcx(x);
    A = y/x;
    A = A';
    D = Sigma - A*Omega*A';
    D = diagrv(eye(n),diag(D));
  
    Sigma = A*Omega*A' + D;
   
    {vol,y,mr,rc,prc} = Compute_RC_Factor(1,Sigma,A,Omega,D);
  endif;

  all_rc[.,iter-1] = rc[1:4];
  all_prc[.,iter-1] = prc[1:4];
  
  iter = iter + 1;
endo;

print ftosa(100*all_prc,"%lf",3,2);


graphset;
  begwind;
  window(2,2,0);

  fonts("simplex simgrma");
  _pdate = ""; _pnum = 2; _pframe = 0; _pgrid = {1,1};
  _pnumht = 0.22; _paxht = 0.20; _ptitlht = 0.275;

  ytics(0,8,2,2);
  let xlab = "MKT" "SMB" "HML" "MOM";
  asclabel(xlab,0);

  setwind(1);
    title("\216EW - S&P 100");
    _pbartyp = {6 1};
    bar(0,100*all_RC[.,1]);

  setwind(2);
    title("\216MV - S&P 100");
    _pbartyp = {6 2};
    bar(0,100*all_RC[.,2]);

  setwind(3);
    title("\216MDP - S&P 100");
    _pbartyp = {6 4};
    bar(0,100*all_RC[.,3]);

  setwind(4);
    title("\216ERC - S&P 100");
    _pbartyp = {6 5};
    bar(0,100*all_RC[.,4]);
    
    graphprt("-c=2 -cf=chap6-2-2-4.eps");

  endwind;

